from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.component import getUtility
from zope.interface import classProvides
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces._content import IContentish, IFolderish

from Products.TinyMCE.libs import json
from Products.TinyMCE.interfaces.utility import ITinyMCE
from Products.TinyMCE.interfaces.utility import ITinyMCELayout
from Products.TinyMCE.interfaces.utility import ITinyMCEToolbar
from Products.TinyMCE.interfaces.utility import ITinyMCELibraries
from Products.TinyMCE.interfaces.utility import ITinyMCEResourceTypes

from OFS.SimpleItem import SimpleItem

def form_adapter(context):
    """Form Adapter"""
    return getUtility(ITinyMCE)

class TinyMCE(SimpleItem):
    """TinyMCE Utility"""
    implements(ITinyMCE)
    classProvides(
        ITinyMCELayout,
        ITinyMCEToolbar,
        ITinyMCELibraries,
        ITinyMCEResourceTypes
        )
    security = ClassSecurityInfo()

    resizing = FieldProperty(ITinyMCELayout['resizing'])
    autoresize = FieldProperty(ITinyMCELayout['autoresize'])
    autoresize_bottom_margin = FieldProperty(ITinyMCELayout['autoresize_bottom_margin'])
    editor_width = FieldProperty(ITinyMCELayout['editor_width'])
    editor_height = FieldProperty(ITinyMCELayout['editor_height'])
    directionality = FieldProperty(ITinyMCELayout['directionality'])
    contextmenu = FieldProperty(ITinyMCELayout['contextmenu'])
    content_css = FieldProperty(ITinyMCELayout['content_css'])
    styles = FieldProperty(ITinyMCELayout['styles'])
    tablestyles = FieldProperty(ITinyMCELayout['tablestyles'])

    toolbar_width = FieldProperty(ITinyMCEToolbar['toolbar_width'])
    
    toolbar_external = FieldProperty(ITinyMCEToolbar['toolbar_external'])

    toolbar_save = FieldProperty(ITinyMCEToolbar['toolbar_save'])

    toolbar_cut = FieldProperty(ITinyMCEToolbar['toolbar_cut'])
    toolbar_copy = FieldProperty(ITinyMCEToolbar['toolbar_copy'])
    toolbar_paste = FieldProperty(ITinyMCEToolbar['toolbar_paste'])
    toolbar_pastetext = FieldProperty(ITinyMCEToolbar['toolbar_pastetext'])
    toolbar_pasteword = FieldProperty(ITinyMCEToolbar['toolbar_pasteword'])

    toolbar_undo = FieldProperty(ITinyMCEToolbar['toolbar_undo'])
    toolbar_redo = FieldProperty(ITinyMCEToolbar['toolbar_redo'])

    toolbar_search = FieldProperty(ITinyMCEToolbar['toolbar_search'])
    toolbar_replace = FieldProperty(ITinyMCEToolbar['toolbar_replace'])

    toolbar_style = FieldProperty(ITinyMCEToolbar['toolbar_style'])

    toolbar_bold = FieldProperty(ITinyMCEToolbar['toolbar_bold'])
    toolbar_italic = FieldProperty(ITinyMCEToolbar['toolbar_italic'])
    toolbar_underline = FieldProperty(ITinyMCEToolbar['toolbar_underline'])
    toolbar_strikethrough = FieldProperty(ITinyMCEToolbar['toolbar_strikethrough'])
    toolbar_sub = FieldProperty(ITinyMCEToolbar['toolbar_sub'])
    toolbar_sup = FieldProperty(ITinyMCEToolbar['toolbar_sup'])

    toolbar_forecolor = FieldProperty(ITinyMCEToolbar['toolbar_forecolor'])
    toolbar_backcolor = FieldProperty(ITinyMCEToolbar['toolbar_backcolor'])

    toolbar_justifyleft = FieldProperty(ITinyMCEToolbar['toolbar_justifyleft'])
    toolbar_justifycenter = FieldProperty(ITinyMCEToolbar['toolbar_justifycenter'])
    toolbar_justifyright = FieldProperty(ITinyMCEToolbar['toolbar_justifyright'])
    toolbar_justifyfull = FieldProperty(ITinyMCEToolbar['toolbar_justifyfull'])

    toolbar_bullist = FieldProperty(ITinyMCEToolbar['toolbar_bullist'])
    toolbar_numlist = FieldProperty(ITinyMCEToolbar['toolbar_numlist'])
    toolbar_outdent = FieldProperty(ITinyMCEToolbar['toolbar_outdent'])
    toolbar_indent = FieldProperty(ITinyMCEToolbar['toolbar_indent'])

    toolbar_tablecontrols = FieldProperty(ITinyMCEToolbar['toolbar_tablecontrols'])

    toolbar_link = FieldProperty(ITinyMCEToolbar['toolbar_link'])
    toolbar_unlink = FieldProperty(ITinyMCEToolbar['toolbar_unlink'])
    toolbar_anchor = FieldProperty(ITinyMCEToolbar['toolbar_anchor'])
    toolbar_image = FieldProperty(ITinyMCEToolbar['toolbar_image'])
    toolbar_media = FieldProperty(ITinyMCEToolbar['toolbar_media'])

    toolbar_charmap = FieldProperty(ITinyMCEToolbar['toolbar_charmap'])
    toolbar_hr = FieldProperty(ITinyMCEToolbar['toolbar_hr'])
    toolbar_advhr = FieldProperty(ITinyMCEToolbar['toolbar_advhr'])
    toolbar_insertdate = FieldProperty(ITinyMCEToolbar['toolbar_insertdate'])
    toolbar_inserttime = FieldProperty(ITinyMCEToolbar['toolbar_inserttime'])
    toolbar_emotions = FieldProperty(ITinyMCEToolbar['toolbar_emotions'])
    toolbar_nonbreaking = FieldProperty(ITinyMCEToolbar['toolbar_nonbreaking'])
    toolbar_pagebreak = FieldProperty(ITinyMCEToolbar['toolbar_pagebreak'])

    toolbar_print = FieldProperty(ITinyMCEToolbar['toolbar_print'])
    toolbar_preview = FieldProperty(ITinyMCEToolbar['toolbar_preview'])
    toolbar_iespell = FieldProperty(ITinyMCEToolbar['toolbar_iespell'])
    toolbar_removeformat = FieldProperty(ITinyMCEToolbar['toolbar_removeformat'])
    toolbar_cleanup = FieldProperty(ITinyMCEToolbar['toolbar_cleanup'])
    toolbar_visualaid = FieldProperty(ITinyMCEToolbar['toolbar_visualaid'])
    toolbar_visualchars = FieldProperty(ITinyMCEToolbar['toolbar_visualchars'])
    toolbar_attribs = FieldProperty(ITinyMCEToolbar['toolbar_attribs'])
    toolbar_code = FieldProperty(ITinyMCEToolbar['toolbar_code'])
    toolbar_fullscreen = FieldProperty(ITinyMCEToolbar['toolbar_fullscreen'])
    
    link_using_uids = FieldProperty(ITinyMCEResourceTypes['link_using_uids'])
    allow_captioned_images = FieldProperty(ITinyMCEResourceTypes['allow_captioned_images'])
    containsobjects = FieldProperty(ITinyMCEResourceTypes['containsobjects'])
    containsanchors = FieldProperty(ITinyMCEResourceTypes['containsanchors'])
    linkable = FieldProperty(ITinyMCEResourceTypes['linkable'])
    imageobjects = FieldProperty(ITinyMCEResourceTypes['imageobjects'])
    customplugins = FieldProperty(ITinyMCEResourceTypes['customplugins'])

    security.declareProtected('View', 'isTinyMCEEnabled')
    def isTinyMCEEnabled(self, useragent='', allowAnonymous=False, REQUEST=None, context=None, fieldName=None):
        if not REQUEST:
            REQUEST = self.REQUEST
        def numerics(s):
            """Convert a string into a tuple of all digit sequences"""
            seq = ['']
            for c in s:
                if c.isdigit():
                    seq[-1] = seq[-1] + c
                elif seq[-1]:
                    seq.append('')
            return tuple([ int(val) for val in seq if val])

        # First check whether the user actually wants kupu
        pm = getToolByName(self, 'portal_membership')
        if pm.isAnonymousUser() and not allowAnonymous:
            return False

        user = pm.getAuthenticatedMember()
        if not pm.isAnonymousUser():
            editor = user.getProperty('wysiwyg_editor')
            if editor and editor.lower() != 'tinymce':
                return False

        # Then check whether the current content allows html
        if context is not None and fieldName and hasattr(context, 'getWrappedField'):
            field = context.getWrappedField(fieldName)
            if field:
                allowedTypes = getattr(field, 'allowable_content_types', None)
                if allowedTypes is not None and not 'text/html' in [t.lower() for t in allowedTypes]:
                    return False

        # Then check whether their browser supports it.
        if not useragent:
            useragent = REQUEST['HTTP_USER_AGENT']

        if 'BEOS' in useragent:
            return False

        def getver(s):
            """Extract a version number given the string which precedes it"""
            pos = useragent.find(s)
            if pos >= 0:
                tail = useragent[pos+len(s):].strip()
                verno = numerics(tail.split(' ')[0])
                return verno
            return None

        try:
            v = getver('Opera/')
            if not v:
                v = getver('Opera ')
            if v:
                return v >= (9,0)

            mozillaver = getver('Mozilla/')
            if mozillaver > (5,0):
                return True
            elif mozillaver == (5,0):
                verno = getver(' rv:')
                if verno:
                    return verno >= (1,3,1)
                verno = getver(' AppleWebKit/')
                if verno:
                    return verno >= (525,1)
                    verno = getver(' Safari/')
                    if verno:
                        return verno >= (522,12)

            verno = getver('MSIE')
            if verno:
                return verno >= (5,5)
        except:
            # In case some weird browser makes the test code blow up.
            pass
        return False

    security.declarePrivate('getEnabledButtons')
    def getEnabledButtons(self, context):
        buttons = []

        # Get enabled buttons from control panel
        if self.toolbar_save:
            try:
                if not context.checkCreationFlag():
                    buttons.append('save')
            except:
                pass

        if self.toolbar_cut:
            buttons.append('cut')
        if self.toolbar_copy:
            buttons.append('copy')
        if self.toolbar_paste:
            buttons.append('paste')

        if self.toolbar_pastetext:
            buttons.append('pastetext')
        if self.toolbar_pasteword:
            buttons.append('pasteword')

        if self.toolbar_undo:
            buttons.append('undo')
        if self.toolbar_redo:
            buttons.append('redo')

        if self.toolbar_search:
            buttons.append('search')
        if self.toolbar_replace:
            buttons.append('replace')

        if self.toolbar_style:
            buttons.append('style')

        if self.toolbar_bold:
            buttons.append('bold')
        if self.toolbar_italic:
            buttons.append('italic')
        if self.toolbar_underline:
            buttons.append('underline')
        if self.toolbar_strikethrough:
            buttons.append('strikethrough')
        if self.toolbar_sub:
            buttons.append('sub')
        if self.toolbar_sup:
            buttons.append('sup')

        if self.toolbar_forecolor:
            buttons.append('forecolor')
        if self.toolbar_backcolor:
            buttons.append('backcolor')

        if self.toolbar_justifyleft:
            buttons.append('justifyleft')
        if self.toolbar_justifycenter:
            buttons.append('justifycenter')
        if self.toolbar_justifyright:
            buttons.append('justifyright')
        if self.toolbar_justifyfull:
            buttons.append('justifyfull')

        if self.toolbar_bullist:
            buttons.append('bullist')
        if self.toolbar_numlist:
            buttons.append('numlist')
        if self.toolbar_outdent:
            buttons.append('outdent')
        if self.toolbar_indent:
            buttons.append('indent')

        if self.toolbar_tablecontrols:
            buttons.append('tablecontrols')

        if self.toolbar_link:
            buttons.append('link')
        if self.toolbar_unlink:
            buttons.append('unlink')
        if self.toolbar_anchor:
            buttons.append('anchor')
        if self.toolbar_image:
            buttons.append('image')
        if self.toolbar_media:
            buttons.append('media')

        if self.toolbar_charmap:
            buttons.append('charmap')
        if self.toolbar_hr:
            buttons.append('hr')
        if self.toolbar_advhr:
            buttons.append('advhr')
        if self.toolbar_insertdate:
            buttons.append('insertdate')
        if self.toolbar_inserttime:
            buttons.append('inserttime')
        if self.toolbar_emotions:
            buttons.append('emotions')
        if self.toolbar_nonbreaking:
            buttons.append('nonbreaking')
        if self.toolbar_pagebreak:
            buttons.append('pagebreak')

        if self.toolbar_print:
            buttons.append('print')
        if self.toolbar_preview:
            buttons.append('preview')
        if self.toolbar_iespell:
            buttons.append('iespell')
        if self.toolbar_removeformat:
            buttons.append('removeformat')
        if self.toolbar_cleanup:
            buttons.append('cleanup')
        if self.toolbar_visualaid:
            buttons.append('visualaid')
        if self.toolbar_visualchars:
            buttons.append('visualchars')
        if self.toolbar_attribs:
            buttons.append('attribs')
        if self.toolbar_code:
            buttons.append('code')
        if self.toolbar_fullscreen:
            buttons.append('fullscreen')

        # Return the buttons
        return buttons

    security.declarePrivate ('translateButtonsFromKupu')
    def translateButtonsFromKupu(self, context, buttons):

        return_buttons = []
        
        for button in buttons:
            if button == 'save-button':
                try:
                    if not context.checkCreationFlag():
                        return_buttons.append('save')
                except:
                    pass
            elif button == 'bg-basicmarkup':
                pass
            elif button == 'bold-button':
                return_buttons.append('bold')
            elif button == 'italic-button':
                return_buttons.append('italic')
            elif button == 'bg-supsuper-button':
                pass
            elif button == 'subscript':
                return_buttons.append('sub')
            elif button == 'supscript':
                return_buttons.append('sup')
            elif button == 'bg-colorchooser':
                pass
            elif button == 'forecolor-button':
                return_buttons.append('forecolor')
            elif button == 'hilitecolor-button':
                return_buttons.append('backcolor')
            elif button == 'bg-justify':
                pass
            elif button == 'justifyleft-button':
                return_buttons.append('justifyleft')
            elif button == 'justifycenter-button':
                return_buttons.append('justifycenter')
            elif button == 'justifyright-button':
                return_buttons.append('justifyright')
            elif button == 'bg-list':
                pass
            elif button == 'list-ol-addbutton':
                return_buttons.append('numlist')
            elif button == 'list-ul-addbutton':
                return_buttons.append('bullist')
            elif button == 'definitionlist':
                pass
            elif button == 'bg-indent':
                pass
            elif button == 'outdent-button':
                return_buttons.append('outdent')
            elif button == 'indent-button':
                return_buttons.append('indent')
            elif button == 'bg-drawers':
                pass
            elif button == 'imagelibdrawer-button':
                return_buttons.append('image')
            elif button == 'linklibdrawer-button' or button == 'linkdrawer-button' or button == 'anchors-button':
                if 'link' not in return_buttons:
                    return_buttons.append('link')
            elif button == 'embed-tab':
                return_buttons.append('media')
            elif button == 'manage-anchors-tab':
                return_buttons.append('anchor')
            elif button == 'toc-tab':
                pass
            elif button == 'tabledrawer-button':
                return_buttons.append('tablecontrols')
            elif button == 'bg-remove':
                pass
            elif button == 'removeimage-button':
                pass
            elif button == 'removelink-button':
                return_buttons.append('unlink')
            elif button == 'bg-undo':
                pass
            elif button == 'undo-button':
                return_buttons.append('undo')
            elif button == 'redo-button':
                return_buttons.append('redo')
            elif button == 'spellchecker':
                return_buttons.append('iespell')
            elif button == 'source':
                return_buttons.append('code')
            elif button == 'styles' or button == 'ulstyles' or 'olstyles':
                if 'style' not in return_buttons:
                    return_buttons.append('style')
            elif button == 'zoom':
                return_buttons.append('fullscreen')
            else:
                if button not in return_buttons:
                    return_buttons.append(button)
        return return_buttons

    security.declarePrivate ('getValidElements')
    def getValidElements(self):

        XHTML_TAGS = set(
            'a abbr acronym address area b base bdo big blockquote body br '
            'button caption cite code col colgroup dd del div dfn dl dt em '
            'fieldset form h1 h2 h3 h4 h5 h6 head hr html i img input ins kbd '
            'label legend li link map meta noscript object ol optgroup option '
            'p param pre q samp script select small span strong style sub sup '
            'table tbody td textarea tfoot th thead title tr tt ul var'.split())

        CORE_ATTRS = set(
            'id title style class'.split())

        I18N_ATTRS = set(
            'lang dir'.split())

        FOCUS_ATTRS = set(
            'accesskey tabindex'.split())

        COMMON_ATTRS = CORE_ATTRS | I18N_ATTRS

        valid_elements = {
            'a': COMMON_ATTRS | FOCUS_ATTRS | set('charset type name href hreflang rel rev shape coords target'.split()),
            'abbr': COMMON_ATTRS.copy(),
            'acronym': COMMON_ATTRS.copy(),
            'address': COMMON_ATTRS.copy(),
            'applet': CORE_ATTRS | set('codebase archive code object alt name width height align hspace vspace'.split()),
            'area': COMMON_ATTRS | FOCUS_ATTRS | set('shape coords href nohref alt target'.split()),
            'b': COMMON_ATTRS.copy(),
            'base': set('id href target'.split()),
            'bdo': CORE_ATTRS | set('lang dir'.split()),
            'big': COMMON_ATTRS.copy(),
            'blockquote': COMMON_ATTRS | set('cite'.split()),
            'body': COMMON_ATTRS | set('background bgcolor text link vlink alink'.split()),
            'br': CORE_ATTRS | set('clear'.split()),
            'button': COMMON_ATTRS | FOCUS_ATTRS | set('name value type disabled'.split()),
            'caption': COMMON_ATTRS | set('align'.split()),
            'center': COMMON_ATTRS.copy(),
            'cite': COMMON_ATTRS.copy(),
            'code': COMMON_ATTRS.copy(),
            'col': COMMON_ATTRS | set('span width align char charoff valign'.split()),
            'colgroup': COMMON_ATTRS | set('span width align char charoff valign'.split()),
            'dd': COMMON_ATTRS.copy(),
            'del': COMMON_ATTRS | set('cite datetime'.split()),
            'dfn': COMMON_ATTRS.copy(),
            'div': COMMON_ATTRS | set('align'.split()),
            'dl': COMMON_ATTRS | set('compact'.split()),
            'dt': COMMON_ATTRS.copy(),
            'em': COMMON_ATTRS.copy(),
            'embed': '*',
            'fieldset': COMMON_ATTRS.copy(),
            'form': COMMON_ATTRS | set('action method name enctype accept accept-charset target'.split()),
            'h1': COMMON_ATTRS | set('align'.split()),
            'h2': COMMON_ATTRS | set('align'.split()),
            'h3': COMMON_ATTRS | set('align'.split()),
            'h4': COMMON_ATTRS | set('align'.split()),
            'h5': COMMON_ATTRS | set('align'.split()),
            'h6': COMMON_ATTRS | set('align'.split()),
            'head': I18N_ATTRS | set('id profile'.split()),
            'hr': COMMON_ATTRS | set('align noshade size width'.split()),
            'html': I18N_ATTRS | set('id xmlns'.split()),
            'i': COMMON_ATTRS.copy(),
            'img': COMMON_ATTRS | set('src alt name longdesc height width usemap ismap<ismap align border hspace vspace'.split()),
            'input': COMMON_ATTRS | FOCUS_ATTRS | set('type name value checked disabled readonly size maxlength src alt usemap accept align'.split()),
            'ins': COMMON_ATTRS | set('cite datetime'.split()),
            'kbd': COMMON_ATTRS.copy(),
            'label': COMMON_ATTRS | FOCUS_ATTRS | set('for'.split()),
            'legend': COMMON_ATTRS | set('accesskey align'.split()),
            'li': COMMON_ATTRS | set('type'.split()),
            'link': COMMON_ATTRS | set('charset href hreflang type rel rev media target'.split()),
            'map': I18N_ATTRS | set('id title name style class'.split()),
            'meta': I18N_ATTRS | set('id http-equiv name content scheme'.split()),
            'noscript': COMMON_ATTRS.copy(),
            'object': COMMON_ATTRS | set('declare classid codebase data type codetype archive standby height width usemap name tabindex align border hspace vspace'.split()),
            'ol': COMMON_ATTRS | set('compact type'.split()),
            'optgroup': COMMON_ATTRS | set('disabled label'.split()),
            'option': COMMON_ATTRS | set('selected disabled label value'.split()),
            'p': COMMON_ATTRS | set('align'.split()),
            'param': set('id name value valuetype type'.split()),
            'pre': COMMON_ATTRS | set('width'.split()),
            'q': COMMON_ATTRS | set('cite'.split()),
            'samp': COMMON_ATTRS.copy(),
            'script': set('id charset type language src defer'.split()),
            'select': COMMON_ATTRS | FOCUS_ATTRS | set('type name value checked disabled readonly size maxlength src alt usemap accept align multiple'.split()),
            'small': COMMON_ATTRS.copy(),
            'span': COMMON_ATTRS.copy(),
            'strong': COMMON_ATTRS.copy(),
            'style': I18N_ATTRS | set('id type media title'.split()),
            'sub': COMMON_ATTRS.copy(),
            'sup': COMMON_ATTRS.copy(),
            'table': COMMON_ATTRS | set('summary width border frame rules cellspacing cellpadding align bgcolor'.split()),
            'tbody': COMMON_ATTRS | set('align char charoff valign'.split()),
            'td': COMMON_ATTRS | set('align char charoff valign bgcolor abbr axis headers scope rowspan colspan nowrap width height'.split()),
            'textarea': COMMON_ATTRS | FOCUS_ATTRS | set('name rows cols disabled readonly'.split()),
            'tfoot': COMMON_ATTRS | set('align char charoff valign'.split()),
            'th': COMMON_ATTRS | set('align char charoff valign bgcolor abbr axis headers scope rowspan colspan nowrap width height'.split()),
            'thead': COMMON_ATTRS | set('align char charoff valign'.split()),
            'title': I18N_ATTRS | set('id'.split()),
            'tr': COMMON_ATTRS | set('align char charoff valign bgcolor'.split()),
            'tt': COMMON_ATTRS.copy(),
            'ul': COMMON_ATTRS | set('compact type'.split()),
            'var': COMMON_ATTRS.copy(),
            }

        # Get safe html transform
        safe_html = getattr(getToolByName(self, 'portal_transforms'), 'safe_html')

        # Get custom tags
        valid_tags = set(safe_html.get_parameter_value('valid_tags'))
        custom_tags = valid_tags - XHTML_TAGS

        # Add custom tags
        for custom_tag in custom_tags:
            if not valid_elements.has_key(custom_tag):
                valid_elements[custom_tag] = COMMON_ATTRS

        # Get kupu library tool
        kupu_library_tool = getToolByName(self, 'kupu_library_tool')

        # Get stripped combinations
        stripped_combinations = kupu_library_tool.get_stripped_combinations()

        # Strip combinations
        for (stripped_combination_tags, stripped_combination_attributes) in stripped_combinations:
            stripped_combination_attributes_set = set(stripped_combination_attributes)
            for stripped_combination_tag in stripped_combination_tags:
                if valid_elements.has_key(stripped_combination_tag):
                    valid_elements[stripped_combination_tag] -= stripped_combination_attributes_set

        # Remove to be stripped attributes
        stripped_attributes = set(kupu_library_tool.get_stripped_attributes())
        style_whitelist = kupu_library_tool.getStyleWhitelist()
        style_attribute = "style"
        if len(style_whitelist) > 0:
            style_attribute = 'style<' + '?'.join(style_whitelist)

        # Remove elements which are not in valid_tags
        for valid_element in valid_elements.keys():
            if valid_element not in valid_tags:
                del valid_elements[valid_element]
            else:
                if valid_elements[valid_element] != '*':
                    valid_elements[valid_element] -= stripped_attributes
                #if 'style' in valid_elements[valid_element]:
                #    valid_elements[valid_element].remove('style')
                #    valid_elements[valid_element].add(style_attribute)

        # Convert sets to lists
        for valid_element in valid_elements.keys():
            valid_elements[valid_element] = sorted(valid_elements[valid_element])

        return valid_elements

    security.declareProtected('View', 'getContentType')
    def getContentType(self, object=None, fieldname=None):
        if hasattr(object, 'getContentType'):
            return object.getContentType(fieldname)
        return 'text/html'

    security.declareProtected('View', 'getConfiguration')
    def getConfiguration(self, context=None, field=None):
        results = {}

        # Get widget attributes
        widget = getattr(field, 'widget', None)
        filter_buttons = getattr(widget, 'filter_buttons', None)
        allow_buttons = getattr(widget, 'allow_buttons', None)
        redefine_parastyles = getattr (widget, 'redefine_parastyles', None)
        parastyles = getattr (widget, 'parastyles', None)

        # Add styles to results
        results['styles'] = []
        results['table_styles'] = []
        if redefine_parastyles is None or not redefine_parastyles:
            for tablestyle in self.tablestyles.split('\n'):
                tablestylefields = tablestyle.split('|');
                results['styles'].append(tablestylefields[0] + '|table|' + tablestylefields[1]);
                results['table_styles'].append(tablestylefields[0] + '=' + tablestylefields[1]);
            results['styles'].extend(self.styles.split('\n'))

        if parastyles is not None:
            results['styles'].extend(parastyles)

        # Get buttons from control panel
        results['buttons'] = self.getEnabledButtons(context=context)

        # Filter buttons
        if allow_buttons is not None:
            allow_buttons = self.translateButtonsFromKupu(context=context, buttons=allow_buttons)
            results['buttons'] = filter(lambda x:x in results['buttons'],allow_buttons)
        if filter_buttons is not None:
            filter_buttons = self.translateButtonsFromKupu(context=context, buttons=filter_buttons)
            results['buttons'] = filter(lambda x:x not in filter_buttons, results['buttons'])

        # Get valid html elements
        results['valid_elements'] = self.getValidElements()

        # Set toolbar_location
        if self.toolbar_external:
            results['toolbar_location'] = 'external'
        else:
            results['toolbar_location'] = 'top'

        if self.autoresize:
            results['path_location'] = 'none'
            results['resizing_use_cookie'] = False
            results['resizing'] = False
            results['autoresize'] = True
        else:
            results['path_location'] = 'bottom'
            results['resizing_use_cookie'] = True
            if self.resizing:
                results['resizing'] = True
            else:
                results['resizing'] = False
            results['autoresize'] = False

        try:
            results['autoresize_bottom_margin'] = int(self.autoresize_bottom_margin)
        except:
            results['autoresize_bottom_margin'] = 40

        if '%' in self.editor_width:
            results['resize_horizontal'] = False
        else:
            results['resize_horizontal'] = True

        try:
            results['editor_width'] = int(self.editor_width)
        except:
            results['editor_width'] = 600

        try:
            results['editor_height'] = int(self.editor_height)
        except:
            results['editor_height'] = 400

        try:
            results['toolbar_width'] = int(self.toolbar_width)
        except:
            results['toolbar_width'] = 440

        results['directionality'] = self.directionality
        if self.contextmenu:
            results['contextmenu'] = True
        else:
            results['contextmenu'] = False

        if self.content_css and self.content_css.strip() != "":
            results['content_css'] = self.content_css
        else:
            results['content_css'] = self.absolute_url() + """/@@tinymce-getstyle"""

        if self.link_using_uids:
            results['link_using_uids'] = True
        else:
            results['link_using_uids'] = False

        if self.allow_captioned_images:
            results['allow_captioned_images'] = True
        else:
            results['allow_captioned_images'] = False

        results['customplugins'] = []
        if self.customplugins is not None:
            results['customplugins'].extend(self.customplugins.split('\n'))

        results['entity_encoding'] = self.entity_encoding

        portal_url = getToolByName(self, 'portal_url')
        results['portal_url'] = portal_url()

        props = getToolByName(self, 'portal_properties')
        livesearch = props.site_properties.getProperty('enable_livesearch', False)
        if livesearch:
            results['livesearch'] = True
        else:
            results['livesearch'] = False

        AVAILABLE_LANGUAGES = set(
           'ar bs ch da el es fa fr he hu ii it ko lv ms nl pl ro sc si sl sr '
           'tr tw vi bg ca cs de en et fi gl hr ia is ja lt mk nb nn pt ru se '
           'sk sq sv tt uk zh'.split())

        if context.REQUEST.has_key('LANGUAGE'):
            results['language'] = context.REQUEST.LANGUAGE[:2]
            if results['language'] not in AVAILABLE_LANGUAGES:
                results['language'] = "en"
        else:
            results['language'] = "en"

        try:
            results['document_url'] = context.absolute_url()
            if context.checkCreationFlag():
                parent = getattr(context.aq_inner, 'aq_parent', None)
                parent = getattr(parent, 'aq_parent', None)
                parent = getattr(parent, 'aq_parent', None)
                results['parent'] = parent.absolute_url() + "/"
            else:
                if IFolderish.providedBy(context):
                    results['parent'] = context.absolute_url() + "/"
                else:
                    results['parent'] = getattr(context.aq_inner, 'aq_parent', None).absolute_url() + "/"
        except:
            results['parent'] = portal_url() + "/"
            results['document_url'] = portal_url()

        return json.write(results)



from zope.interface import implements
from zope.component import adapts
from Products.TinyMCE.libs import json
from Products.TinyMCE.interfaces.utility import ITinyMCE

from Products.TinyMCE.adapters.interfaces.JSONSearch import IJSONSearch
from Products.CMFCore.interfaces._content import IContentish, IFolderish
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.interfaces import INonStructuralFolder
from Products.CMFPlone import utils
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_inner
	
class JSONSearch(object):
	"""Returns a list of search results in JSON"""
	implements(IJSONSearch)

	def __init__(self, context):
		"""Constructor"""
		self.context = context

	def getInfoFromBrain(self, brain):
		"""Gets information from a brain id, url, portal_type, title, icon, is_folderish"""

		id = brain.getId
		uid = brain.UID
		url = brain.getURL()
		portal_type = brain.portal_type
		title = brain.Title
		icon = brain.getIcon
		is_folderish = brain.is_folderish

		return {
		'id': id,
		'uid': uid,
		'url': url,
		'portal_type': portal_type,
		'title' : title,
		'icon' : icon,
		'is_folderish' : is_folderish
		}

	def getSearchResults(self, filter_meta_types, searchtext):
		"""Returns the actual search result"""

		catalog_results = []
		results = {}

		results['parent_url'] = ''
		results['path'] = []

		if searchtext:
			for brain in self.context.portal_catalog.searchResults({'SearchableText':'%s*' % searchtext, 'meta_type':filter_meta_types, 'sort_on':'sortable_title'}):
				catalog_results.append(self.getInfoFromBrain(brain));

		# add catalog_ressults
		results['items'] = catalog_results; 

		# return results in JSON format
		return json.write(obj=results, escaped_forward_slash=True);




from zope.interface import Interface

class IJSONDetails(Interface):
	"""Return details of the current object in JSON"""

	def __init__(self, context):
		"""Constructor"""

	def getDetails(self):
		"""Returns the actual details"""



from zope.interface import implements
from zope.component import adapts
from Products.TinyMCE.libs import json
from Products.TinyMCE.interfaces.utility import ITinyMCE

from Products.TinyMCE.adapters.interfaces.JSONFolderListing import IJSONFolderListing
from Products.CMFCore.interfaces._content import IContentish, IFolderish
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.interfaces import INonStructuralFolder
from Products.CMFPlone import utils
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_inner
	
class JSONFolderListing(object):
	"""Returns a folderish like listing in JSON"""
	implements(IJSONFolderListing)
	#adapts(IContentish, IPloneSiteRoot);

	def __init__(self, context):
		"""Constructor"""
		self.context = context
		
	def getBreadcrums(self):
		"""Get breadcrums"""
		#TODO: getToolByName is deprecated
		ptool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IPropertiesTool');
		utool = getToolByName(self.context, 'portal_url');
		portal_url = utool();
		result = [];

		# SiteRoot
		result.append({'title':ptool.title(),'url':portal_url})
		
		relative = utool.getRelativeContentPath(self.context);
		portal = utool.getPortalObject();

		for i in range(len(relative)):
			now = relative[ :i+1 ]
			obj = aq_inner(portal.restrictedTraverse(now));

			if IFolderish.providedBy(obj):
				if not now[-1] == 'talkback':
					result.append({'title':obj.Title(),'url':portal_url + '/' + '/'.join(now)})
		return result;		
		
	def getInfoFromBrain(self, brain):
		"""Gets information from a brain id, url, portal_type, title, icon, is_folderish"""

		id = brain.getId;
		uid = brain.UID;
		url = brain.getURL();
		portal_type = brain.portal_type;
		title = brain.Title;
		icon = brain.getIcon;
		is_folderish = brain.is_folderish;

		return {
		'id': id,
		'uid': uid,
		'url': url,
		'portal_type': portal_type,
		'title' : title,
		'icon' : icon,
		'is_folderish' : is_folderish
		}

	def getListing(self, filter_meta_types):
		"""Returns the actual listing"""

		catalog_results = [];
		results = {};

		object = aq_inner(self.context);

		if IPloneSiteRoot.providedBy(object):
			results['parent_url'] = '';
		else:
			results['parent_url'] = object.getParentNode().absolute_url();

		# get all items from siteroot to context (title and url)
		results['path'] = self.getBreadcrums();
		
		
		# check if object is a folderish object, if not, get it's parent.
		if not IFolderish.providedBy(object):
			object = object.getParentNode();
		
		# get all meta types and get information from brains
		for brain in object.getFolderContents({'meta_type':filter_meta_types, 'sort_on':'sortable_title'}):
			catalog_results.append(self.getInfoFromBrain(brain));

		# add catalog_ressults
		results['items'] = catalog_results; 

		# return results in JSON format
		return json.write(obj=results, escaped_forward_slash=True);




from zope.interface import implements
from zope.component import adapts
from zope.component import getUtility
from Products.TinyMCE.libs import json
from Products.TinyMCE.interfaces.utility import ITinyMCE

from Products.TinyMCE.adapters.interfaces.JSONDetails import IJSONDetails
from Products.CMFCore.interfaces._content import IContentish, IFolderish
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_inner
from elementtree import HTMLTreeBuilder;
	
class JSONDetails(object):
	"""Return details of the current object in JSON"""
	implements(IJSONDetails)

	def __init__(self, context):
		"""Constructor"""
		self.context = context

	def getDetails(self):
		"""Returns the actual details"""

		utility = getUtility(ITinyMCE)
		anchor_meta_types = utility.containsanchors.split('\n')
		image_meta_types = utility.imageobjects.split('\n')

		results = {};
		results['title'] = self.context.Title()
		results['description'] = self.context.Description()

		if self.context.meta_type in image_meta_types:
			results['thumb'] = self.context.absolute_url() + "/image_thumb"
		else:
			results['thumb'] = ""

		if self.context.meta_type in anchor_meta_types:
			results['anchors'] = [];
			# content = '<html><head></head><body><a name="tildeancor"></a></body></html>'
			tree = HTMLTreeBuilder.TreeBuilder()
			tree.feed('<root>%s</root>' % self.context.getText())
			rootnode = tree.close()
			for x in rootnode.getiterator():
				if x.tag == "a":
					if "name" in x.keys():
						results['anchors'].append(x.attrib['name'])
		else:
			results['anchors'] = []

		return json.write(obj=results, escaped_forward_slash=True);



from zope.interface import Interface

class ISave(Interface):
	"""Saves the richedit field"""

	def __init__(self, context):
		"""Constructor"""

	def save(self, text, fieldname):
		"""Saves the specified richedit field"""



from zope.interface import Interface

class ITinyMCEBrowserView(Interface):
	"""TinyMCE Browser View"""

	def upload(self):
		"""Upload a file to the zodb"""

	def save(self, text, fieldname):
		"""Saves the specified richedit field"""

	def jsonLinkableFolderListing(self):
		"""Returns the folderlisting of linkable objects in JSON"""

	def jsonImageFolderListing(self):
		"""Returns the folderlisting of image objects in JSON"""

	def jsonLinkableSearch(self):
		"""Returns the search results of linkable objects in JSON"""

	def jsonImageSearch(self):
		"""Returns the search results of image objects in JSON"""

	def jsonDetails(self):
		"""Returns the details of an object in JSON"""



from Products.CMFCore.utils import getToolByName
from zope.interface import implements
from zope.component import getUtility
from Products.Five.browser import BrowserView

from Products.TinyMCE.browser.interfaces.url import ITinyMCEUrl

class TinyMCEUrl(BrowserView):
	"""TinyMCE Url"""
	implements(ITinyMCEUrl)

	def getPathByUID(self):
		"""Returns the path of an object specified in the request by UID"""

		context = self.context
		request = context.REQUEST

		if not hasattr(request, 'uid'):
			return ""

		uid = request['uid']

		reference_tool = getToolByName(context, 'reference_catalog')
		obj = reference_tool.lookupObject(uid)

		if obj:
			return obj.absolute_url()

		return ""



from zope.interface import Interface

class IJSONSearch(Interface):
	"""Returns a list of search results in JSON"""

	def __init__(self, context):
		"""Constructor"""

	def getSearchResults(self, filter_meta_types):
		"""Returns the actual search results"""

	def getInfoFromBrain(self, brain):
		"""
		Gets information from a brain
		id, url, portal_type, title, icon, is_folderish
		"""



"""Test setup for integration and functional tests."""

from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

@onsetup
def setup_product():
    """Set up the package and its dependencies."""
    
    fiveconfigure.debug_mode = True
    import Products.TinyMCE
    zcml.load_config('configure.zcml', Products.TinyMCE)
    fiveconfigure.debug_mode = False
    
setup_product()
ptc.setupPloneSite(products=['Products.TinyMCE'])

class TinyMCETestCase(ptc.PloneTestCase):
    """We use this base class for all the tests in this package. If necessary,
    we can put common utility or setup code in here. This applies to unit 
    test cases.
    """

class TinyMCEFunctionalTestCase(ptc.FunctionalTestCase):
    """We use this class for functional integration tests that use doctest
    syntax. Again, we can put basic common utility or setup code in here.
    """



from Products.PythonScripts.standard import url_quote
from Products.CMFCore.utils import getToolByName
from zope.interface import implements
from zope.component import getUtility
from Products.Five.browser import BrowserView
from Acquisition import aq_inner

from Products.TinyMCE.browser.interfaces.style import ITinyMCEStyle
from Products.TinyMCE.interfaces.utility import ITinyMCE

class TinyMCEStyle(BrowserView):
	"""TinyMCE Style"""
	implements(ITinyMCEStyle)

	def getStyle(self):
		"""Returns a stylesheet with all content styles"""
		
		registry = getToolByName(aq_inner(self.context), 'portal_css')
		registry_url = registry.absolute_url()
		context = aq_inner(self.context)

		styles = registry.getEvaluatedResources(context)
		skinname = url_quote(aq_inner(self.context).getCurrentSkinName())
		result = []
		#result.append('html { font-size: 69%; }')
		#result.append('body { width: }')

		for style in styles:
			if style.getMedia() not in ('print', 'projection') and style.getRel()=='stylesheet' :
				src = "<!-- @import url(%s/%s/%s); -->" % (registry_url, skinname, style.getId())
				result.append(src)
		return "\n".join(result)




from zope.interface import implements
from zope.component import getUtility
from Products.Five.browser import BrowserView
from Acquisition import aq_inner

from Products.TinyMCE.adapters.interfaces.JSONFolderListing import IJSONFolderListing;
from Products.TinyMCE.adapters.interfaces.JSONSearch import IJSONSearch;
from Products.TinyMCE.adapters.interfaces.JSONDetails import IJSONDetails;
from Products.TinyMCE.adapters.interfaces.Upload import IUpload;
from Products.TinyMCE.adapters.interfaces.Save import ISave;
from Products.TinyMCE.browser.interfaces.browser import ITinyMCEBrowserView
from Products.TinyMCE.interfaces.utility import ITinyMCE

class TinyMCEBrowserView(BrowserView):
	"""TinyMCE Browser View"""
	implements(ITinyMCEBrowserView)

	def upload(self):
		"""Upload a file to the zodb"""

		context = aq_inner(self.context)
		object = IUpload(self.context)
		return object.upload()
	
	def save(self, text, fieldname):
		"""Saves the specified richedit field"""

		context = aq_inner(self.context)
		object = ISave(self.context)
		return object.save(text, fieldname)

	def jsonLinkableFolderListing(self):
		"""Returns the folderlisting of linkable objects in JSON"""

		utility = getUtility(ITinyMCE)
		linkable_meta_types = utility.linkable.split('\n')

		context = aq_inner(self.context)
		object = IJSONFolderListing(self.context)
		return object.getListing(linkable_meta_types)
	
	def jsonImageFolderListing(self):
		"""Returns the folderlisting of image objects in JSON"""

		utility = getUtility(ITinyMCE)
		image_meta_types = utility.imageobjects.split('\n')
		image_meta_types.extend(utility.containsobjects.split('\n'))

		context = aq_inner(self.context)
		object = IJSONFolderListing(self.context)
		return object.getListing(image_meta_types)

	def jsonLinkableSearch(self, searchtext):
		"""Returns the search results of linkable objects in JSON"""

		utility = getUtility(ITinyMCE)
		linkable_meta_types = utility.linkable.split('\n')

		context = aq_inner(self.context)
		object = IJSONSearch(self.context)
		return object.getSearchResults(linkable_meta_types, searchtext)
	
	def jsonImageSearch(self, searchtext):
		"""Returns the search results of image objects in JSON"""

		utility = getUtility(ITinyMCE)
		image_meta_types = utility.imageobjects.split('\n')
		image_meta_types.extend(utility.containsobjects.split('\n'))

		context = aq_inner(self.context)
		object = IJSONSearch(self.context)
		return object.getSearchResults(image_meta_types, searchtext)

	def jsonDetails(self):
		"""Returns the details of an object in JSON"""
	
		context = aq_inner(self.context)
		object = IJSONDetails(self.context)
		return object.getDetails()



import unittest
from Products.TinyMCE.tests.base import TinyMCETestCase
from Products.TinyMCE.interfaces.utility import ITinyMCE
from zope.component import getUtility

class TestUtility(TinyMCETestCase):
    """Test the utility"""
    
    def afterSetUp(self):
        """Run before each test"""
        self.utility = getUtility(ITinyMCE)
        
    def beforeTearDown(self):
        """Run after each test"""

    def test_isTinyMCEEnabled(self):
        isenabled = self.utility.isTinyMCEEnabled
        self.failUnless(isenabled)

def test_suite():
    """Setup test suite"""

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUtility))
    return suite



from Products.TinyMCE.tests.base import TinyMCETestCase
from Products.TinyMCE.interfaces.utility import ITinyMCE
from zope.component import getUtility
from zope.testing import doctestunit
from Testing import ZopeTestCase as ztc
import doctest
import unittest

def test_suite():
    """This sets up a test suite that actually runs the tests in transforms_integration.txt"""
    return unittest.TestSuite([
        # The actual test is in transforms_integration.txt
        ztc.ZopeDocFileSuite(
            'tests/transforms_integration.txt', package='Products.TinyMCE',
            test_class=TinyMCETestCase),
        ])



from zope.interface import implements
from zope.component import adapts;
from Products.CMFCore.utils import getToolByName
from Products.PythonScripts.standard import html_quote, newline_to_br

from Products.TinyMCE.interfaces.utility import ITinyMCE
from zope.component import getUtility
from Products.TinyMCE.adapters.interfaces.Upload import IUpload
from Products.CMFCore.interfaces._content import IContentish, IFolderish;
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone import utils
from Acquisition import aq_inner

TEMPLATE = """
<html>
<head></head>
<body onload="%s"></body>
</html>
"""

class Upload(object):
	"""Adds the uploaded file to the folder"""
	implements(IUpload);

	def __init__(self, context):
		"""Constructor"""

		self.context = context;

	def errorMessage(self, msg):
		"""Returns an error message"""
	
		script = TEMPLATE % ("window.parent.uploadError('" + msg.replace("'", "\\'") + "');")
		return script
	
	def okMessage(self, msg):
		"""Returns an ok message"""

		script = TEMPLATE % ("window.parent.uploadOk('" + msg.replace("'", "\\'") + "');")
		return script

	def cleanupFilename(self, name):
		"""Generate a unique id which doesn't match	the system generated ids"""

		context = self.context
		id = ''
		name = name.replace('\\', '/') # Fixup Windows filenames
		name = name.split('/')[-1] # Throw away any path part.
		for c in name:
			if c.isalnum() or c in '._':
				id += c

		# Raise condition here, but not a lot we can do about that
		if context.check_id(id) is None and getattr(context,id,None) is None:
			return id

		# Now make the id unique
		count = 1
		while 1:
			if count==1:
				sc = ''
			else:
				sc = str(count)
			newid = "copy%s_of_%s" % (sc, id)
			if context.check_id(newid) is None and getattr(context,newid,None) is None:
				return newid
			count += 1

	def upload(self):
		"""Adds uploaded file"""

		object = aq_inner(self.context);
		if not IFolderish.providedBy(object):
			object = object.getParentNode()

		context = self.context
		request = context.REQUEST
		ctr_tool = getToolByName(self.context, 'content_type_registry')
		id = request['uploadfile'].filename
		
		content_type = request['uploadfile'].headers["Content-Type"]
		typename = ctr_tool.findTypeName(id, content_type, "")
	
		# Permission checks based on code by Danny Bloemendaal

		# 1) check if we are allowed to create an Image in folder 
		if not typename in [t.id for t in context.getAllowedTypes()]: 
			return self.errorMessage("Not allowed to upload a file of this type to this folder")

		# 2) check if the current user has permissions to add stuff 
		if not context.portal_membership.checkPermission('Add portal content',context): 
			return self.errorMessage("You do not have permission to upload files in this folder")

		# Get an unused filename without path
		id = self.cleanupFilename(id)

		newid = context.invokeFactory(type_name=typename, id=id,
			title=request['uploadtitle'],
			description=request['uploaddescription'],
		)

		if newid is None or newid == '':
			newid = id 

		obj = getattr(context,newid, None)
		if typename == 'Image':
			obj.setImage(request['uploadfile'])
		else:
			obj.setFile(request['uploadfile'])

		if not obj:
			return self.errorMessage("Could not upload the file")

		obj.reindexObject()

		utility = getUtility(ITinyMCE)
		if utility.link_using_uids:
			return self.okMessage("resolveuid/%s" % (obj.UID()))
		return self.okMessage("%s" % (obj.absolute_url()))



from zope.interface import Interface

class IUpload(Interface):
	"""Adds the uploaded file to the folder"""

	def __init__(self, context):
		"""Constructor"""

	def errorMessage(self, msg):
		"""Returns an error message"""

	def okMessage(self, msg):
		"""Returns an ok message"""

	def cleanupFilename(self, name):
		"""Generate a unique id which doesn't match	the system generated ids"""

	def upload(self):
		"""Adds uploaded file"""